#ifndef _SYMTAB_H_
#define _SYMTAB_H_

#define SYMTAB_SIZE     128


#define SF_FORWARD	(1 << 0)	/* Was this forward reference */
					/* in a call or goto?         */
#define SF_ABS		(1 << 1)	/* Absolute symbol; i.e. EQU */
#define SF_MOD		(1 << 2)	/* Modifiable; i.e. SET */
#define SF_SCOPE_LOCAL	(0)		/* Symbol has local psect visibility */
#define SF_SCOPE_FILE	(1 << 3)	/* Symbol has file visibility */
#define SF_SCOPE_GLOBAL	(1 << 4)	/* Symbol has global visibility */
#define SF_DECLARED	(1 << 5)	/* Symbol delcared (beyond 'global' */
					/* or 'file' directive)             */

struct symlist {
  struct symlist *next;
  char *label;
  int val;
  int psect;	/* psect identifier */
  unsigned int flags;
  char *file;	/* File where defined (or first encountered?) */
  char *pgroup;	/* Psect group identifier; faster to store than look up */
  int size;	/* # of words/bytes associated w/symbol */
};

/* Hash of 128 symlists. */
extern struct symlist *symtabs[SYMTAB_SIZE];

extern int allglobal;	/* All symbols are declared global? */

extern void symInit(void);
extern void symDestroy(void);
extern void symAdd(const char *buf, const int loc, unsigned int f);
extern int symModify(const char *lbl, const int loc, unsigned int f);
extern int symLookup(const char *lbl);
extern int symInScope(const struct symlist *sp, const char *lbl);
extern void symSetScope(const char *lbl, const int scope);
extern void symSetSize(const int loc);
extern int symFlags(const char *lbl);
extern void symDump(FILE *lfp);

extern int symResolveAddr(const char *op);
extern int symResolveLiteral(const char *op);

#define resolve_addr symResolveAddr
// #define resolve_literal symResolveLiteral


#endif
